# coding: utf-8
# author: gabriel couture
from typing import List, Dict

from pyorthanc.datastructure.tree.series import Series
from pyorthanc.orthanc import Orthanc


class Study:
    """Represent an study that is in an Orthanc server

    This object has many getters that allow the user to retrieve metadata
    or the entire DICOM file of the Series
    """

    def __init__(
            self, study_identifier: str,
            orthanc: Orthanc,
            study_information: Dict = None) -> None:
        """Constructor

        Parameters
        ----------
        study_identifier
            Orthanc study identifier.
        orthanc
            Orthanc object.
        study_information
            Dictionary of study's information.
        """
        self.orthanc = orthanc

        self.identifier = study_identifier
        self.information = study_information

        self.series: List[Series] = []

    def get_identifier(self) -> str:
        """Get Study identifier

        Returns
        -------
        str
            Study identifier
        """
        return self.identifier

    def get_main_information(self) -> Dict:
        """Get Study information

        Returns
        -------
        Dict
            Dictionary of study information
        """
        if self.information is None:
            self.information = self.orthanc.get_study_information(
                self.identifier
            )

        return self.information

    def get_referring_physician_name(self) -> str:
        """Get referring physician name

        Returns
        -------
        str
            Referring physician Name.
        """
        return self.get_main_information()['MainDicomTags']['ReferringPhysicianName']

    def get_date(self) -> str:
        """Get study date

        Returns
        -------
        str
            Study date
        """
        return self.get_main_information()['MainDicomTags']['StudyDate']

    def get_time(self) -> str:
        """Get Study time

        Returns
        -------
        str
            Study time
        """
        return self.get_main_information()['MainDicomTags']['StudyTime']

    def get_id(self) -> str:
        """Get Study ID

        Returns
        -------
        str
            Study ID
        """
        return self.get_main_information()['MainDicomTags']['StudyID']

    def get_parent_patient_identifier(self) -> str:
        """Get the Orthanc identifier of the parent patient

        Returns
        -------
        str
            Parent patient's identifier.
        """
        return self.get_main_information()['ParentPatient']

    def get_patient_information(self) -> Dict:
        """Get patient information

        Returns
        -------
        Dict
            Patient general information.
        """
        return self.get_main_information()['PatientMainDicomTags']

    def get_series(self) -> List[Series]:
        """Get Study series

        Returns
        -------
        List[Series]
            List of study's Series
        """
        return self.series

    def build_series(self) -> None:
        """Build a list of the study's series
        """
        series_identifiers = self.orthanc.get_study_series_information(
            self.identifier
        )

        self.series = list(map(
            lambda i: Series(i['ID'], self.orthanc),
            series_identifiers
        ))

    def __str__(self):
        return f'Study (id={self.get_id()}, identifier={self.get_identifier()})'

    def trim(self) -> None:
        """Delete empty series
        """
        self.series = list(filter(
            lambda series: not series.is_empty(),
            self.series
        ))

    def is_empty(self) -> bool:
        """Check if series is empty

        Returns
        -------
        bool
            True if study has no instance
        """
        return self.series == []



# coding: utf-8
# author: Gabriel Couture
import unittest

from pyorthanc import Orthanc
from pyorthanc.datastructure import Patient
from tests.integration import setup_server
from tests.integration.data import a_patient


class TestPatient(unittest.TestCase):

    @classmethod
    def setUpClass(cls) -> None:
        global orthanc_subprocess
        orthanc_subprocess = setup_server.setup_orthanc_server()

    @classmethod
    def tearDownClass(cls) -> None:
        global orthanc_subprocess
        setup_server.stop_orthanc_server_and_remove_data_directory(orthanc_subprocess)
        del orthanc_subprocess

    def setUp(self) -> None:
        setup_server.setup_data()
        self.patient = Patient(
            a_patient.IDENTIFIER,
            Orthanc(setup_server.ORTHANC_URL)
        )

    def tearDown(self) -> None:
        self.patient = None
        setup_server.clear_data()

    def test_givenAPatient_whenGettingMainInformation_thenResultIsExpectedPatientInformation(self):
        keys_to_exclude = {'LastUpdate'}

        result = self.patient.get_main_information()

        self.assertEqual(
            {key: value for key, value in result.items() if key not in keys_to_exclude},
            {key: value for key, value in a_patient.INFORMATION.items() if key not in keys_to_exclude},
        )

    def test_givenAPatient_whenGettingPatientID_thenResultIsExpectedPatientID(self):
        result = self.patient.get_id()

        self.assertEqual(result, a_patient.ID)

    def test_givenAPatient_whenGettingPatientName_thenResultIsExpectedPatientName(self):
        result = self.patient.get_patient_name()

        self.assertEqual(result, a_patient.NAME)

    def test_givenAPatient_whenGettingPatientSex_thenResultIsExpectedSex(self):
        result = self.patient.get_patient_sex()

        self.assertEqual(result, a_patient.SEX)

    def test_givenAPatient_whenBuildingStudies_thenPatientHasStudies(self):
        self.patient.build_studies()

        self.assertEqual(
            len(self.patient.get_studies()),
            len(a_patient.STUDIES)
        )

    def test_givenAPatientWithEmptyStudies_whenTrimPatient_thenEmptyStudiesGetsDeleted(self):
        self.patient.build_studies()  # When getting building, studies do not get built by default
        expected_number_of_study = 0

        self.patient.trim()

        self.assertEqual(
            expected_number_of_study,
            len(self.patient.get_studies())
        )



# coding: utf-8
# author: gabriel couture
from typing import Dict

from pyorthanc import Orthanc


class RemoteModality:
    """Wrapper around Orthanc API when dealing with a (remote) modality.
    """

    def __init__(self, orthanc: Orthanc, modality: str) -> None:
        """Constructor

        Parameters
        ----------
        orthanc
            Orthanc object.
        modality
            Remote modality.
        """
        self.orthanc: Orthanc = orthanc
        self.modality: str = modality

    def echo(self) -> bool:
        """C-Echo to remote modality

        Returns
        -------
        bool
            True if C-Echo succeeded.
        """
        return self.orthanc.echo_to_modality(self.modality)

    def query(self, data: Dict) -> Dict:
        """C-Find (Querying with data)

        Parameters
        ----------
        data
            Dictionary to send in the body of request.

        Returns
        -------
        requests.Response

        Examples
        -------
        >>> data = {'Level': 'Study',
        ...         'Query': {
        ...             'PatientID':'03HD*',
        ...             'StudyDescription':'*Chest*',
        ...             'PatientName':''
        ...         }
        ... }

        >>> remote_modality = RemoteModality(
        ...     orthanc=Orthanc('http://localhost:8042'),
        ...     modality='sample'
        ... )

        >>> remote_modality.query(data)
        """
        return self.orthanc.query_on_modality(self.modality, data=data)

    def retrieve(self, data: Dict) -> bool:
        """Retrieve (C-Move) to local modality

        Parameters
        ----------
        data
            Dictionary to send in the body of request.

        Returns
        -------
        bool
            True if the C-Move operation was sent without problem, else False.
        """
        return self.orthanc.move_from_modality(self.modality, data=data)

    def move(self, query_identifier: str, cmove_data: Dict) -> bool:
        """C-Move query results to another modality

        C-Move SCU: Send all the results to another modality whose AET is in the body

        Parameters
        ----------
        query_identifier
            Query identifier.
        cmove_data
            Ex. {'TargetAET': 'modality_name', "Synchronous": False}

        Returns
        -------
        bool
            True if the C-Move operation was sent without problem, else False.

        Examples
        --------
        >>> remote_modality = RemoteModality(Orthanc('http://localhost:8042'), 'modality')
        >>> query_id = remote_modality.query(
        ...     data={'Level': 'Study',
        ...           'Query': {'QueryRetrieveLevel': 'Study',
        ...                     'Modality':'SR'}}).json()

        >>> remote_modality.move(
        ...     query_identifier=query_id['ID'],
        ...     cmove_data={'TargetAET': 'modality'}
        ... )

        """
        return self.orthanc.move_query_results_to_given_modality(
            query_identifier,
            data=cmove_data
        )



# coding: utf-8
# author: Gabriel Couture

IDENTIFIER = '118bc493-b3b3172a-082119bd-f6802ec3-81695613'
PARENT_PATIENT_IDENTIFIER = 'e34c28ce-981b0e5c-2a481559-cf0d5fbe-053335f8'
PATIENT_MAIN_INFORMATION = {
    'PatientBirthDate': '',
    'PatientID': '03HDQ000',
    'PatientName': 'MR-R',
    'PatientSex': 'M'
}
DATE = '20100223'
REFERRING_PHYSICIAN_NAME = ''
ID = ''
TIME = ''
SERIES = [
    'c4c1fcc9-ae63f793-40cbcf25-fbd3efe5-ad72ff06',
    'e2a7df26-99673e0f-05aa84cd-c89677c0-634a2a96',
    '60108266-ece4d8f7-7b028286-a7b61f25-c6d33f0b'
]
INFORMATION = {
    'ID': IDENTIFIER,
    'IsStable': False,
    'LastUpdate': '20190913T185658',
    'MainDicomTags': {
        'AccessionNumber': '',
        'ReferringPhysicianName': '',
        'StudyDate': DATE,
        'StudyID': ID,
        'StudyInstanceUID': '1.2.840.113619.2.55.3.671782272.419.1266452812.128',
        'StudyTime': TIME
    },
    'ParentPatient': PARENT_PATIENT_IDENTIFIER,
    'PatientMainDicomTags': PATIENT_MAIN_INFORMATION,
    'Series': SERIES,
    'Type': 'Study'
}



# coding: utf-8
# author: Gabriel Couture

IDENTIFIER = 'c4c1fcc9-ae63f793-40cbcf25-fbd3efe5-ad72ff06'
INSTANCES = ['22dcf059-8fd3ade7-efb39ca3-7f46b248-0200abc9']
MODALITY = 'RTDOSE'
MANUFACTURER = 'ADAC'
PARENT_STUDY = '118bc493-b3b3172a-082119bd-f6802ec3-81695613'
INFORMATION = {
    'ExpectedNumberOfInstances': None,
    'ID': IDENTIFIER,
    'Instances': INSTANCES,
    'IsStable': False,
    'LastUpdate': '20190916T131815',
    'MainDicomTags': {
        'ImageOrientationPatient': '1\\0\\0\\0\\1\\0',
        'Manufacturer': MANUFACTURER,
        'Modality': MODALITY,
        'SeriesInstanceUID': '2.16.840.1.113669.2.931128.981194414.20100301170148.767969',
        'SeriesNumber': '',
        'StationName': 'pinnc-2'
    },
    'ParentStudy': PARENT_STUDY,
    'Status': 'Unknown',
    'Type': 'Series'
}



# coding: utf-8
# author: gabriel couture
import unittest

from pyorthanc import Orthanc
from tests.integration import setup_server
from tests.integration.data import a_patient


class TestOrthancPatientDeleters(unittest.TestCase):

    @classmethod
    def setUpClass(cls) -> None:
        global orthanc_subprocess
        orthanc_subprocess = setup_server.setup_orthanc_server()

    @classmethod
    def tearDownClass(cls) -> None:
        global orthanc_subprocess
        setup_server.stop_orthanc_server_and_remove_data_directory(orthanc_subprocess)
        del orthanc_subprocess

    def setUp(self) -> None:
        self.orthanc = Orthanc(setup_server.ORTHANC_URL)

    def tearDown(self) -> None:
        self.orthanc = None
        setup_server.clear_data()

    def given_patient_in_orthanc_server(self):
        setup_server.setup_data()

    def test_givenOrthancWithPatient_whenDeletingPatientData_thenResultIsTrue(self):
        self.given_patient_in_orthanc_server()

        result = self.orthanc.delete_patient(a_patient.IDENTIFIER)

        self.assertTrue(result)

    def test_givenOrthancWithoutPatient_whenDeletingPatientData_thenResultIsFalse(self):
        result = self.orthanc.delete_patient(a_patient.IDENTIFIER)

        self.assertFalse(result)



# coding: utf-8
# author: Gabriel Couture
import unittest
from datetime import datetime

from pyorthanc import Orthanc
from pyorthanc.datastructure import Instance
from tests.integration import setup_server
from tests.integration.data import a_instance


class TestInstance(unittest.TestCase):

    @classmethod
    def setUpClass(cls) -> None:
        global orthanc_subprocess
        orthanc_subprocess = setup_server.setup_orthanc_server()

    @classmethod
    def tearDownClass(cls) -> None:
        global orthanc_subprocess
        setup_server.stop_orthanc_server_and_remove_data_directory(orthanc_subprocess)
        del orthanc_subprocess

    def setUp(self) -> None:
        setup_server.setup_data()

        self.instance = Instance(
            a_instance.IDENTIFIER,
            Orthanc(setup_server.ORTHANC_URL)
        )

    def tearDown(self) -> None:
        self.instance = None
        setup_server.clear_data()

    def test_givenAInstance_whenGettingMainInformation_thenResultIsExpectedPatientInformation(self):
        keys_to_exclude = {'LastUpdate', 'FileSize', 'FileUuid'}

        result = self.instance.get_main_information()

        self.assertDictEqual(
            {key: value for key, value in result.items() if key not in keys_to_exclude},
            {key: value for key, value in a_instance.INFORMATION.items() if key not in keys_to_exclude},
        )

    def test_givenAInstance_whenGettingFileSize_thenResultIsAInt(self):
        result = self.instance.get_file_size()

        self.assertIsInstance(result, int)  # The fileSize value depends on the machine where Orthanc run

    def test_givenAInstance_whenGettingCreationDate_thenResultIsExpectedCreationDate(self):
        expected_date = datetime(
            year=2010,
            month=3,
            day=1,
            hour=17,
            minute=1,
            second=55
        )

        result = self.instance.get_creation_date()

        self.assertEqual(result, expected_date)

    def test_givenAInstance_whenGettingParentSeriesIdentifier_thenResultIsExpectedParentSeriesIdentifier(self):
        result = self.instance.get_parent_series_identifier()

        self.assertEqual(result, a_instance.PARENT_SERIES_IDENTIFIER)



import subprocess

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

FLAKE8_COMMAND = ['./venv/bin/flake8', '--ignore=E501', 'pyorthanc', 'tests']
MYPY_COMMAND = ['./venv/bin/mypy', 'pyorthanc']
UNIT_TESTS_COMMAND = ['./venv/bin/python', '-m', 'unittest', 'discover', '-s', './tests/unit/']
INTEGRATION_TESTS_COMMAND = ['./venv/bin/python', '-m', 'unittest', 'discover', '-s', './tests/integration']


def _run_command(command: str) -> None:
    try:
        subprocess.check_call(command)

    except subprocess.CalledProcessError as error:
        print('Command failed with exit code', error.returncode)
        exit(error.returncode)


class UnitTests(TestCommand):
    description = 'run unit tests'
    user_options = []

    def run_tests(self):
        _run_command(UNIT_TESTS_COMMAND)


class IntegrationTests(TestCommand):
    description = 'run integration tests'
    user_options = []

    def run_tests(self):
        _run_command(INTEGRATION_TESTS_COMMAND)


class LintTests(TestCommand):
    description = 'run linters'
    user_options = []

    def run_tests(self):
        _run_command(FLAKE8_COMMAND)
        _run_command(MYPY_COMMAND)


class AllTests(TestCommand):
    description = 'run unit tests, integration tests and linters'
    user_options = []

    def run_tests(self):
        _run_command(UNIT_TESTS_COMMAND)
        _run_command(INTEGRATION_TESTS_COMMAND)
        _run_command(FLAKE8_COMMAND)
        _run_command(MYPY_COMMAND)


with open('./README.md', 'r') as file_hanlder:
    long_description = file_hanlder.read()


setup(
    name='pyorthanc',
    version='0.2.4',
    packages=find_packages(),
    url='https://gitlab.physmed.chudequebec.ca/gacou54/pyorthanc',
    license='MIT',
    author='Gabriel Couture',
    author_email='gacou54@gmail.com',
    description='Orthanc REST API python wrapper with additional utilities',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=['requests'],
    cmdclass={
        'lint': LintTests,
        'unit': UnitTests,
        'integration': IntegrationTests,
        'test': AllTests
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Healthcare Industry',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.7'
    ]
)
